#include "Sio.h"
#include <stdio.h>

#include <ctype.h>
#include <libc.h>

extern "C" {
	int tgetent(char*, const char*);
	int tgetflag(const char*);
	int tgetnum(const char*);
	char *tgetstr(const char*, char**);
	char *tgoto(const char*, int, int);
	char *tparm(char*, int ...);
	void tputs(const char*, int, int(*)(int));
}

int RJS_Sio::initialized=0;
int RJS_Sio::current_line=0;

int outch(int i)
{
cout << char(i);
return 0;
}

int RJS_Sio::num_cols() 
{ 
	return cols; 
}

int RJS_Sio::num_lines() 
{ 
	return lines; 
}

void RJS_Sio::noecho()
{
  ctty.sg_flags &= ~ECHO;
  ioctl(fileno(stdin),TIOCSETN, &ctty);
}

void RJS_Sio::echo()
{
  ctty.sg_flags |= ECHO;
  ioctl(fileno(stdin),TIOCSETN, &ctty);
}

void RJS_Sio::cooked()
{
  ctty.sg_flags &= ~CBREAK;
  ioctl(fileno(stdin),TIOCSETN, &ctty);
}

void RJS_Sio::cbreak()
{
  ctty.sg_flags |= CBREAK;
  ioctl(fileno(stdin),TIOCSETN, &ctty);
}

void RJS_Sio::set_page_mode()
{
	page_mode=Page;
}

void RJS_Sio::clear_page_mode()
{
	page_mode=NoPage;
}

int RJS_Sio::get_page_mode()
{
	return page_mode;
}

void RJS_Sio::set_current_line(int l) 
{
if (l>=0 && l<=lines) current_line=l;
}

int RJS_Sio::get_current_line() 
{
  return current_line;
}

void RJS_Sio::init()
{
	ion=0;
	uon=0;
	in_word=0;
	cols=80;
	lines=24;
	blink_on=NULL;
	bold_on=NULL;
	cls=NULL;
	delete_char=NULL;
	delete_line=NULL;
	erase_to_end_of_line=NULL;
	erase_to_end_of_screen=NULL;
	home=NULL;
	inverse_on=NULL;
	inverse_off=NULL;
	plain=NULL;
	restore_cursor=NULL;
	save_cursor=NULL;
	underline_on=NULL;
	underline_off=NULL;
}


static char *bp;

char *GetTermcap(const char *s)
{
char *temp=bp;
   tgetstr(s,&bp);
   return temp;
	
}


RJS_Sio::RJS_Sio(SioMode mode)
{

if (initialized) return;
else { init(); initialized++;}

char temp[1024];
char *term=getenv("TERM");
	
	bp=buffer;

	page_mode=mode;

	ioctl(fileno(stdin),TIOCGETP, &savetty);
	ctty=savetty;

	if (!term) term="dumb";
	if (tgetent(temp,term)!=1) return;

	cols=tgetnum("co");
	if (cols==-1) cols=80;
	lines=tgetnum("li");
	if (cols==-1) lines=24;

	blink_on=GetTermcap("mb");
	bold_on=GetTermcap("md");
	cls=GetTermcap("cl");
	delete_char=GetTermcap("dc");
	delete_line=GetTermcap("dl");
	erase_to_end_of_line=GetTermcap("ce");
	erase_to_end_of_screen=GetTermcap("cd");
	home=GetTermcap("ho");
	inverse_on=GetTermcap("so");
	inverse_off=GetTermcap("se");
	plain=GetTermcap("me");
	restore_cursor=GetTermcap("rc");
	save_cursor=GetTermcap("sc");
	underline_on=GetTermcap("us");
	underline_off=GetTermcap("ue");
}

void RJS_Sio::get_tty(struct sgttyb &tty)
{
  ioctl(fileno(stdin),TIOCGETP, &tty);
}

void RJS_Sio::set_tty(struct sgttyb &tty)
{
  ioctl(fileno(stdin),TIOCSETN, &tty);
}

RJS_Sio::~RJS_Sio()
{
  ioctl(fileno(stdin),TIOCSETN, &savetty);
}

int RJS_Sio::more()
{
char ch;
struct sgttyb tty;
int status=1;

  get_tty(tty);

  if (!ion) tputs(inverse_on,1,outch);
  noecho(); cbreak();

  cout << "--MORE--";

getit:
    cin.get(ch);
    switch(ch) {
	case '\n':
		current_line--;
		break;
	case ' ':
		current_line=0;
		break;
	case '\t':
		current_line /= 2;
		break;
	case 'q':
	case 'Q':
		current_line=0; 
		status=0;
		break;
	default:
		cout << '\007';
		goto getit;
    }


  if (!ion) tputs(inverse_off,1,outch);
  cout << "\r        \r";
  set_tty(tty);

  return status;
}

int RJS_Sio::printl(const char *s)
{
  int status=print(s);
  if (status) status=print("\n");
  return status;
}

int RJS_Sio::print(const char *s)
{
  while(*s) {
     if (*s!='\\') {
	if (in_word && ( !isalnum(*s) && *s!='_')) {
		in_word=0; 
		tputs(plain,1,outch);
	}
	cout<< *s;
	if (page_mode==Page && *s=='\n') current_line++;
     }
     else switch(*++s) {
	case 'b': tputs(bold_on,1,outch); break;
	case 'c': tputs(cls,1,outch); current_line=0; break;
	case 'd': tputs(delete_char,1,outch); break;
	case 'e': tputs(erase_to_end_of_line,1,outch); break;
	case 'f': cout << '\014'; break;		// ctrl-l
	case 'g': cout << '\007'; break;		// bell
	case 'h': tputs(home,1,outch); break;
	case 'i': 
		ion = ion ? 0 : 1;
		if (ion) tputs(inverse_on,1,outch);
		else tputs(inverse_off,1,outch);
		break;
	case 'l': tputs(blink_on,1,outch); break;
	case 'n': 
		cout << '\n'; 
		if (page_mode==Page && *s=='\n') current_line++;
		break;		// newline
	case 'p': tputs(plain,1,outch); ion=0; uon=0; break;
	case 'r': cout << '\r'; break;		// carriage return
	case 't': cout << '\t'; break;		// horizontal tab
	case 'u':				// toggle underline
		uon = uon ? 0 : 1;
		if (uon) tputs(underline_on,1,outch);
		else tputs(underline_off,1,outch);
		break;
	case 'v': cout << '\v'; break;		// vertical tab
	case 'B': in_word=1; tputs(bold_on,1,outch);	break;
	case 'D': tputs(delete_line,1,outch);		break;
	case 'E': tputs(erase_to_end_of_screen,1,outch);break;
	case 'I': in_word=1; tputs(inverse_on,1,outch);	break;
	case 'L': in_word=1; tputs(blink_on,1,outch);	break;
	case 'R': tputs(restore_cursor,1,outch);	break;
	case 'S': tputs(save_cursor,1,outch);		break;
	case 'U': in_word=1; tputs(underline_on,1,outch); break;
	default:
		cout << *s;
     }
     if (page_mode==Page && current_line==lines-1) {
        if (!more()) return 0;
     }
     s++;
  }
  return 1;
}
